/*****************************************************************************
 *
 * CRC Demo for PIC24.
 *
 *****************************************************************************
 * FileName:        CRC.c
 * Dependencies:    
 * Processor:       
 * Compiler:       	
 * Linker:          
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Vidyadhar           30/June/06	...	
 
 *****************************************************************************/

#include <p24fJ128GA010.h>    


unsigned int CRCResult;
unsigned int DataCRC[12];

CRCInit()
{
	//*************************************************
	//	STEP 1:
	//	In CRCCON, Configure The Polynomial Length (PLEN)
	//*************************************************/


	//********************************************************************
	//	STEP 2:
	//	In CRCXOR, Configure For The Polynomial x^16 + x^15 + x^2 + 1
	//********************************************************************/

	
	//********************************************************************
	//	STEP 3:
	//	Clear CRCWDAT
	//********************************************************************/


} // End of init_CRC()




CalculateCRC(unsigned int *Src, char Len)
{
	int i, CRCTemp;
	char Temp=0;

	CRCWDAT=0x0000;

	for(i=0;i<Len;i++)
	{	
		CRCDAT=Src[i];				// Input recieved data

		if(CRCCONbits.CRCFUL)
		{

			//********************************************************************
			//	STEP 4:
			//	In CRCCON, Enable The CRC Generator 
			//********************************************************************/
  

		}
		while(CRCCONbits.CRCFUL);	// Wait for available FIFO register
	}
	CRCDAT = 0x0000;  				//Pad last word with 0's

	while (!IFS4bits.CRCIF);		// check for interrupt flag bit
	IFS4bits.CRCIF = 0;

	while(!CRCCONbits.CRCMPT);		// wait for CRC shifter to clear FIFO
	CRCResult = CRCWDAT;			// Ouptut result
	CRCWDAT=0;

}



DisplayWaitCRC()
{
	LCDClear();
	LCDHome();       // Set cursor to Line 1 of LCD
	LCDPut('W');
	LCDPut('a');
	LCDPut('i');
	LCDPut('t');
	LCDPut('i');
	LCDPut('n');
	LCDPut('g');
	LCDPut(' ');
	LCDPut('f');
	LCDPut('o');
	LCDPut('r');
	LCDPut(' ');
	LCDPut('D');
	LCDPut('a');
	LCDPut('t');
	LCDPut('a');
}

DisplayCRCMatch()
{
	LCDClear();
	LCDHome();       // Set cursor to Line 1 of LCD
	LCDPut('C');
	LCDPut('R');
	LCDPut('C');
	LCDPut(' ');
	LCDPut('V');
	LCDPut('e');
	LCDPut('r');
	LCDPut('i');
	LCDPut('f');
	LCDPut('i');
	LCDPut('e');
	LCDPut('d');
	LCDPut(' ');
	LCDPut('O');
	LCDPut('K');
}

DisplayCRCNoMatch()
{
	LCDClear();
	LCDHome();       // Set cursor to Line 1 of LCD
	LCDPut('C');
	LCDPut('R');
	LCDPut('C');
	LCDPut(' ');
	LCDPut('V');
	LCDPut('e');
	LCDPut('r');
	LCDPut('i');
	LCDPut('f');
	LCDPut('i');
	LCDPut('e');
	LCDPut('d');
	LCDPut(' ');
	LCDPut('N');
	LCDPut('O');
	LCDPut('K');
}

main()
{
	LCDInit();           // Initialize LCD module.

	CRCInit();			 // Initialize CRC module. You write this function!
	
    RS232Init();		 // Initialize UART Communications

    DisplayWaitCRC();	 // Display waiting message to LCD.

	while(1)
	{
	    GetRS232(DataCRC, 11);  // Get data message from UART
	
	    CalculateCRC(DataCRC, 11); // Calculate CRC with recieved data
	
		// Display result to LCD.
	    if (CRCResult == 0)
		  DisplayCRCMatch();
		else
		  DisplayCRCNoMatch();
	}
	
}



/********************************************************************************************/
